from .readable_window import ReadableWindow


class StreamWindow(ReadableWindow):
    """
    a basic window that displays the stream from a stream receiver
    
    :type self.wrap_object: gbvision.StreamReceiver
    """
