from gbvision.utils.cameras.camera_data import CameraData

# LIFECAM_STUDIO = CameraData(648.5256168410046, 0.340394, 0, is_immutable=True, name='LIFECAM_STUDIO')

LIFECAM_3000 = CameraData(697.0395744431028, 0.4435563306578366, 0.3337068395920225, is_immutable=True,
                          name='LIFECAM_3000')

UNKNOWN_CAMERA = CameraData(0, 0, 0, is_immutable=True, name='UNKNOWN_CAMERA')
