from .base_shape import BaseShape
from .base_circle import BaseCircle
from .base_rect import BaseRect
from .base_convex_polygon import BaseConvexPolygon
from .base_contour import BaseContour
from .base_rotated_rect import BaseRotatedRect
from .base_polygon import BasePolygon
